#include "memcard.hpp"
#include "config.h"
#include <iostream>
#include <cxxopts.hpp>
#include <string>

#if !defined(STRINGIZE)
#	define STRINGIZE_IMPL(s) #s
#	define STRINGIZE(s) STRINGIZE_IMPL(s)
#endif

namespace {
	constexpr const char* project_ver() {
		return STRINGIZE(PROJECT_VERSION_MAJOR) "."
			STRINGIZE(PROJECT_VERSION_MINOR) "."
			STRINGIZE(PROJECT_VERSION_PATCH);
	}
} //unnamed namespace

int main(int argc, char* argv[]) {
	using std::string;

	cxxopts::Options options(PROJECT_NAME, "PSX memory card inspector");
	options.add_options()
		("h,help", "Show this help and quit")
		("version", "Print version info")
		("i,input", "Path to the input memory card file", cxxopts::value<string>()->default_value(""))
	;
	auto command = options.parse(argc, argv);

	if (command.count("help")) {
		std::cout << options.help() << std::endl;
		return 0;
	}

	if (command.count("version")) {
		std::cout << PROJECT_NAME << " v" << project_ver()
			<< " built with " COMPILER_NAME " " COMPILER_VERSION
			<< std::endl;
		return 0;
	}

	std::string mc_path = command["input"].as<string>();
	if (mc_path.empty()) {
		std::cerr << "No input memory card specified, run with --help for usage\n";
		return 2;
	}

	print_blocks(mc_path);
	return 0;
}
