#pragma once

#include "memcard/block.hpp"
#include <type_traits>
#include <iterator>

namespace mc::psx {
class MemoryCard;

template <bool Const>
class BlockIterator {
	friend class BlockIterator<not Const>;
	typedef typename std::conditional<Const, const MemoryCard, MemoryCard>::type* MemoryCardPtr;
public:
	typedef std::random_access_iterator_tag iterator_category;
	typedef std::size_t size_type;
	typedef std::ptrdiff_t difference_type;
	typedef BasicBlock<Const> value_type;
	typedef BasicBlock<Const> reference;
	typedef BasicBlock<Const> pointer;

	BlockIterator (MemoryCardPtr mc, size_type index);
	BlockIterator (const BlockIterator&) = default;
	BlockIterator (BlockIterator&&) = default;
	BlockIterator();

	template <bool Const2>
	bool operator== (const BlockIterator<Const2>& other) const;
	template <bool Const2>
	bool operator!= (const BlockIterator<Const2>& other) const;
	template <bool Const2>
	bool operator< (const BlockIterator<Const2>& other) const;

	reference operator*();

	BlockIterator& operator++();
	BlockIterator operator++(int);
	BlockIterator& operator--();
	BlockIterator operator--(int);
	template <bool Const2>
	difference_type operator- (const BlockIterator<Const2>& other);
	BlockIterator operator+ (std::ptrdiff_t other) const;
	BlockIterator operator- (std::ptrdiff_t other) const;

private:
	MemoryCardPtr m_mc;
	size_type m_index;
};
} //namespace mc::psx
