#pragma once

#include "memcard/block.hpp"
#include "memcard/content_info.hpp"
#include "memcard/block_iterator.hpp"
#include <vector>
#include <cstdint>
#include <array>
#include <cstddef>
#include <algorithm>

namespace mc::psx {
class MemoryCard {
public:
	typedef BlockIterator<false> iterator;
	typedef BlockIterator<true> const_iterator;

	MemoryCard();
	template <typename IT> explicit MemoryCard (IT beg, IT end);
	~MemoryCard();

	Block operator[] (std::size_t index);
	ConstBlock operator[] (std::size_t index) const;
	ContentInfo content_info() const;

	std::size_t size() const;

	iterator begin();
	iterator end();
	const_iterator cbegin() const;
	const_iterator begin() const;
	const_iterator cend() const;
	const_iterator end() const;

private:
	std::vector<uint8_t> m_data;
};

std::vector<std::size_t> block_group (const MemoryCard& mc, std::size_t index);

template <typename IT>
MemoryCard::MemoryCard (IT beg, IT end) :
	MemoryCard()
{
	std::copy_if(
		beg,
		end,
		m_data.begin(),
		[count = m_data.size()](auto&&) mutable {return static_cast<bool>(count--);}
	);
}

} //namespace mc::psx
