#include "memcard/block_iterator.hpp"
#include "memcard/memorycard.hpp"
#include <cassert>

namespace mc::psx {
template <bool Const>
BlockIterator<Const>::BlockIterator() :
	m_mc(nullptr),
	m_index(0)
{
}

template <bool Const>
BlockIterator<Const>::BlockIterator (MemoryCardPtr mc, size_type index) :
	m_mc(mc),
	m_index(index)
{
}

template <bool Const>
template <bool Const2>
bool BlockIterator<Const>::operator== (const BlockIterator<Const2>& other) const {
	assert(other.m_mc == this->m_mc);
	return this->m_index == other.m_index;
}

template <bool Const>
template <bool Const2>
bool BlockIterator<Const>::operator!= (const BlockIterator<Const2>& other) const {
	assert(other.m_mc == this->m_mc);
	return this->m_index != other.m_index;
}

template <bool Const>
template <bool Const2>
bool BlockIterator<Const>::operator< (const BlockIterator<Const2>& other) const {
	assert(other.m_mc == this->m_mc);
	return this->m_index < other.m_index;
}

template <bool Const>
auto BlockIterator<Const>::operator*() -> reference {
	assert(m_mc);
	return (*m_mc)[m_index];
}

template <bool Const>
BlockIterator<Const>& BlockIterator<Const>::operator++() {
	++m_index;
	return *this;
}

template <bool Const>
BlockIterator<Const> BlockIterator<Const>::operator++(int) {
	auto cpy = *this;
	++(*this);
	return cpy;
}

template <bool Const>
BlockIterator<Const>& BlockIterator<Const>::operator--() {
	--m_index;
	return *this;
}

template <bool Const>
BlockIterator<Const> BlockIterator<Const>::operator--(int) {
	auto cpy = *this;
	--(*this);
	return cpy;
}

template <bool Const>
template <bool Const2>
auto BlockIterator<Const>::operator- (const BlockIterator<Const2>& other) -> difference_type {
	return this->m_index - other.m_index;
}

template <bool Const>
BlockIterator<Const> BlockIterator<Const>::operator+ (std::ptrdiff_t other) const {
	auto retval = *this;
	retval.m_index += other;
	return retval;
}

template <bool Const>
BlockIterator<Const> BlockIterator<Const>::operator- (std::ptrdiff_t other) const {
	auto retval = *this;
	retval.m_index -= other;
	return retval;
}

template class BlockIterator<true>;
template class BlockIterator<false>;
template bool BlockIterator<true>::operator==(const BlockIterator<true>&) const;
template bool BlockIterator<true>::operator==(const BlockIterator<false>&) const;
template bool BlockIterator<false>::operator==(const BlockIterator<true>&) const;
template bool BlockIterator<false>::operator==(const BlockIterator<false>&) const;
template bool BlockIterator<true>::operator!=(const BlockIterator<true>&) const;
template bool BlockIterator<true>::operator!=(const BlockIterator<false>&) const;
template bool BlockIterator<false>::operator!=(const BlockIterator<true>&) const;
template bool BlockIterator<false>::operator!=(const BlockIterator<false>&) const;
template bool BlockIterator<true>::operator<(const BlockIterator<true>&) const;
template bool BlockIterator<true>::operator<(const BlockIterator<false>&) const;
template bool BlockIterator<false>::operator<(const BlockIterator<true>&) const;
template bool BlockIterator<false>::operator<(const BlockIterator<false>&) const;
} //namespace mc::psx
