#include "memcard/memorycard.hpp"
#include "memcard/block.hpp"
#include <iterator>
#include <cassert>

namespace mc::psx {
namespace {
	const constexpr std::size_t MemoryCardSize = 8192 * 16;
} //unnamed namespace

MemoryCard::MemoryCard() :
	m_data(MemoryCardSize)
{
}

MemoryCard::~MemoryCard() = default;

Block MemoryCard::operator[] (std::size_t index) {
	assert(index < 15);
	return Block(m_data.data() + (index + 1) * Block::size(), index);
}

ConstBlock MemoryCard::operator[] (std::size_t index) const {
	assert(index < 15);
	return ConstBlock(m_data.data() + (index + 1) * Block::size(), index);
}

ContentInfo MemoryCard::content_info() const {
	return {ConstBlock(m_data.data(), ConstBlock::ToCBlockIndex)};
}

std::size_t MemoryCard::size() const {
	return std::count_if(
		const_iterator(this, 0),
		const_iterator(this, 15),
		[](const auto& blk) {return blk.has_magic();}
	);
}

auto MemoryCard::begin() -> iterator {
	return iterator(this, 0);
}

auto MemoryCard::end() -> iterator {
	return iterator(this, size());
}

auto MemoryCard::cbegin() const -> const_iterator {
	return const_iterator(this, 0);
}

auto MemoryCard::begin() const -> const_iterator {
	return const_iterator(this, 0);
}

auto MemoryCard::cend() const -> const_iterator {
	return const_iterator(this, size());
}

auto MemoryCard::end() const -> const_iterator {
	return const_iterator(this, size());
}

std::vector<std::size_t> block_group (const MemoryCard& mc, std::size_t index) {
	std::vector<std::size_t> retval;

	auto block = mc[index];
}
} //namespace mc::psx
